package com.yidian.shenghuoquan.newscontent.ui.setting

import androidx.lifecycle.MutableLiveData
import com.orhanobut.hawk.Hawk
import com.yidian.common.AppConfig
import com.yidian.common.BuildConfig
import com.yidian.common.HawkConfig
import com.yidian.common.http.CommonDataSource
import com.yidian.common.http.bean.CommitFeedbackBean
import com.yidian.common.http.callback.ICommitFeedbackCallback
import com.yidian.common.mvvm.BaseViewModel

/**
 * author: yinjiacheng
 * date: 7/16/21 10:27 PM
 * description: 用户反馈
 */
class FeedbackViewModel : BaseViewModel(), ICommitFeedbackCallback {

  /**
   * 用户输入的反馈字数
   */
  val feedbackWordCount by lazy { MutableLiveData(0) }

  /**
   * 用户输入的反馈内容
   */
  val feedbackContent by lazy { MutableLiveData<String>() }

  /**
   * 提交成功事件
   */
  val commitSuccessEvent by lazy { MutableLiveData<Unit>() }

  /**
   * 提交失败事件
   */
  val commitFailureEvent by lazy { MutableLiveData<Unit>() }

  /**
   * 请求提交反馈信息
   */
  fun requestCommitFeedback() {
    // 拼接获取到的用户信息
    val description = "uid:${Hawk.get(HawkConfig.UserId, 0L)},appversion:${BuildConfig.VERSION_NAME},platform:${AppConfig.feedbackPlatform}"
    CommonDataSource.commitFeedback(
      this, CommitFeedbackBean.CommitFeedbackRequest(
        CommitFeedbackBean.Fields(
          CommitFeedbackBean.Assignee(AppConfig.feedbackAssignee),
          description,
          CommitFeedbackBean.Issuetype(AppConfig.feedbackIssueType),
          CommitFeedbackBean.Priority(AppConfig.feedbackPriority),
          CommitFeedbackBean.Project(AppConfig.feedbackProject),
          feedbackContent.value ?: AppConfig.feedbackDefaultSummary
        )
      )
    )
  }

  override fun commitFeedbackSuccess(result: CommitFeedbackBean.CommitFeedbackResponse?) {
    // 提交成功
    commitSuccessEvent.value = Unit
  }

  override fun commitFeedbackFailure(message: String?) {
    // 提交失败
    commitFailureEvent.value = Unit
  }
}
